/**
 * Author: Andreas Gasper
 */

:- multifile query_description/3.
:- multifile metric_/3.
:- multifile metric_attributes/2.

query_description(max_depth_of_nesting, metric, 'Maximal nesting depth of a method').
metric_attributes(max_depth_of_nesting, int(range(0,infinity),defaultthresholds(0,8))).

metric_(max_depth_of_nesting, MethodID, MaxDepth):-
	methodDef(MethodID, ClassID, _ , _, _, _, _),
	not(externT(ClassID)),
    get_max_depth_of_method__(MethodID, MaxDepthOfList),
    MaxDepth is max(1, MaxDepthOfList)
    .

/**
 * get_max_depth_of_method__(+MethodID, -MaxDepthOfList)
 *
 * Succeeds iff MaxDepthOfList is the is maximal nesting depth in the method with MethodID 
 */    

get_max_depth_of_method__(MethodID, MaxDepthOfList):-    
    findall(Depth,
    		(
    			method_contains_control_statement(MethodID, StatementID),
    			depth_of_statement__(StatementID, Depth)
    		),
    		ListOfDepth),
    max_list(ListOfDepth, MaxDepthOfList),
    !
    .

get_max_depth_of_method__(_, 0).


/**
 * depth_of_statement__(+StatementID, -Depth)
 *
 * Succeeds if Depth is the nesting depth of the statement with the StatementID
 */

depth_of_statement__(StatementID, Depth):-
    depth_of_element__(StatementID, 0, Depth).


/**
 * depth_of_element__(+StatementID, +OldDepth, -Depth)
 *
 * Succeeds if Depth is the nesting depth of the element with the StatementID.
 * OldDepth is the depth of the parent statement
 */ 
   
depth_of_element__(StatementID, OldDepth, Depth):-
    ast_parent(StatementID, ParentStatementID),
    ast_node_type_value(ParentStatementID, ParentType),
    is_method__(ParentType),
    Depth is OldDepth + 1
    .

%Switch, no blockT
depth_of_element__(StatementID, OldDepth, NewDepth):-
    ast_parent(StatementID, ParentCandidate),
    is_controlflow_affecting(ParentCandidate),
    Depth is OldDepth + 1,
    depth_of_element__(ParentCandidate, Depth, NewDepth)
    .

depth_of_element__(StatementID, OldDepth, Depth):-
    ast_parent(StatementID, ParentCandidate),
    ast_node_type_value(ParentCandidate, ParentType),
    is_block__(ParentType),
    depth_of_element__(ParentCandidate, OldDepth, Depth)
    .

/**********************************/
is_block__('blockT').

is_method__('methodDef').   