:- module( term_occurrence_computation, [
												cached_occurrences_of_term_in_context/3,
												cached_occurrences_of_all_terms_in_context/2,
												term_occurrences_in_context/4,
												term_counts_in_context/3,
												
												generate_occurrence_counters/0
												
											]).

:- dynamic cached_occurrences_of_term_in_context/3.     % (Context,Term,Count)
:- dynamic cached_occurrences_of_all_terms_in_context/2.% (Context,TotalCountInContext

:- dynamic term_occurrences_in_context/4.
:- dynamic term_counts_in_context/3.

:- use_module('../util/termcache_utils').
:- use_module(term_model).

% ================================================================

generate_occurrence_counters:- % % 1,344,464 inferences, 1.188 CPU in 1.206 seconds (98% CPU, 1132180 Lips)
  cleanup__,                                % 62,544 inferences, 0.063 CPU in 0.061 seconds (102% CPU, 1000704 Lips)
  generate_term_occurrence_counters,        % 1,136,483 inferences, 1.078 CPU in 1.079 seconds (100% CPU, 1054129 Lips)
  generate_term_occurrence_counters_total.  % 145,450 inferences, 0.047 CPU in 0.049 seconds (96% CPU, 3102933 Lips)

cleanup__:-
    retractall(cached_occurrences_of_term_in_context(_,_,_)),
    retractall(cached_occurrences_of_all_terms_in_context(_,_)).
    
generate_term_occurrence_counters :-
    relevant_context_element(Context), 
	context_has_term(Context, Term, _),
    not(cached_occurrences_of_term_in_context(Context,Term,_)),
      count_occurrences_of_term_in_context(Context,Term,Count),
      assert( cached_occurrences_of_term_in_context(Context,Term,Count) ),
     
   fail.
generate_term_occurrence_counters.




	 
/**
 * count_occurrences_of_term_in_context(+Context, ?Term, ?Count)
 *  Arg1 is the ID of a package, class or other program entitity 
 *       in which we want to count term occurrences.
 *  The predicate creates "cached_term_occurences(Context, Term, Count)" 
 *  facts for the Context given in Arg1. Each Term occurring in 
 *  Context yields one fact. The count includes duplicate occurrences
 *  of the Term within the same entity. For instance, we count two
 *  occurrences of "get" in "getTheGetMethod".
 */ 
count_occurrences_of_term_in_context(Context, Term, Count) :-
    bagof( Id,
           context_has_term(Context, Term, Id),
           BAG
    ),
    length(BAG, Count).


% ================================================================     
generate_term_occurrence_counters_total :-
   relevant_context_element(Context), 
   not(cached_occurrences_of_all_terms_in_context(Context,_) ),
     count_occurrences_of_all_terms_in_context(Context, CountOfAllTermsInContext),
     assert( cached_occurrences_of_all_terms_in_context(Context,CountOfAllTermsInContext) ),
   fail.
generate_term_occurrence_counters_total.


count_occurrences_of_all_terms_in_context(Context, CountOfAllTermsInContext):-
   findall(Occurences,
          cached_occurrences_of_term_in_context(Context,_,Occurences),
          OccurenceList),
   sumall( OccurenceList,CountOfAllTermsInContext).  

											