:- module( cultivate_concepts_term_tokenization, [
            create_term_cache/0

   ] ).

% ================================================================   
:- ensure_loaded(utils/borrowed_from_ctc).
:- use_module('../util/termcache_utils').
:- use_module('../util/identifier').
:- use_module(term_model).
% ================================================================


      
      
%==================================================================
create_term_cache :-
     clear_term_cache,                         
     generate_term_id_mapping__,                
     generate_relevant_context_elements__,      
     generate_term_context_mapping__,           
    % cultivate_concepts_terms:report_statistics,     
     true.   
	
% ================================================================

generate_term_id_mapping__ :-
   % find identifier name
    identifier_declaration_or_reference(Id, Name),
    ast_node_type_value(Id,Type),
    not( is_extern(Type,Id) ),
      % split the name
      identifier_built_of_terms(Name, Terms),
      % store association of terms to IDs 
      store_terms__(Id,1,Terms),
      assert_term( lexical_compound(Id,Terms)),
   fail.
generate_term_id_mapping__.   

 

    
   
/**
*
*
*/   
store_terms__(__Id,__Position,[]).
store_terms__(Id,Position,[Term|Tail]) :-
   atomic_list_concat(['stm',Id,'p',Position],TermId),
   assert_term( lexical_term(TermId,Term)),
   assert_term( element_has_lexical(Id,TermId,Position)),
   assert_term( ri_element_has_lexical(TermId, Id)), % with duplicates!
   assert_term_unique( element_has_term(Id, Term)),
   assert_term_unique( ri_element_has_term(Term,Id)),
   NextPosition is Position + 1, 
   store_terms__(Id,NextPosition,Tail). 


  
% ================================================================
generate_relevant_context_elements__ :-
   context_element_type(Type),
   ast_node_term(Type,Language,Template),        % ST
   ast_node_for_id(Context, Language, Template),
   not( is_extern(Type,Context) ),
     assert_term(relevant_context_element(Context)),
   fail.    
generate_relevant_context_elements__.






% ================================================================

generate_term_context_mapping__ :- 
   ri_element_has_lexical(TermId, Id), % det (-,-) 
   element_context(Id,Context),  % (+,-)
       lexical_term(TermId,Term),
       %JN 2010-10-31 should be done after normalization  or replaced by predicate including lemma
       assert_term( context_has_term(Context,Term,Id) ),  % (+,+,+) 
       assert_term( context_has_lexical(Context,TermId,Id) ),  % (+,+)
    fail.
generate_term_context_mapping__.  

    

/**
* element_context__(Id,Ancestor)/2
* Arg2 is a context element of the element Arg1
**/
element_context(Id,Ancestor) :-
    self_or_ancestor(Id,Ancestor),
    relevant_context_element(Ancestor), 
    ast_node_type_value(Ancestor,Type),  % ST: get node type
    context_element_type(Type).


% Arg1 is an AST element type whose instances may be contexts: 
context_element_type(packageT).
context_element_type(classT).
context_element_type(methodT).
context_element_type(constructorT).
%context_element_type(fieldT).
%context_element_type(paramT).

/**
* self_or_ancestor__(Id,Ancestor)/2
* Arg2 is either Arg1 or an ancestor of Arg1
**/
self_or_ancestor(Id,Id). 
self_or_ancestor(Id,Ancestor) :- 
    ast_ancestor(Id,Ancestor) .


%external_package(Package) :-
%    packageT(Package, _),
%    forall(package_contains_type(Package, Type), externT(Type)).

 % ? +     
is_extern(packageT,Id) :- external_package(Id).
is_extern(classT,Id) :- externT(Id).
is_extern(Type,Id) :-  
    not( Type==packageT ),
    not( Type==classT ),
    once( (
      ast_ancestor(Id,Anc),
      ast_node_type_value(Anc,classT)
    ) ),
    externT(Anc). 
      
is_intern(packageT,Id) :- internal_package(Id).
is_intern(classT,Id):- internal_type(Id).
is_intern(Type,Id) :-
   not( Type==packageT ),
   not( Type==classT ),
      ast_node_term(Type,Language,Template),        % ST
      ast_node_for_id(Id, Language, Template),
      ast_ancestor(Id,Anc),
      ast_node_type_value(Anc,classT),
	  internal_type(Anc)  
    .
/* =============================================================
   Performance measurements on JHotDraw / 273 classes, 
   dual core 2 GHz, 4 GB RAM
   =============================================================
   
TIME OF PREVIOUS VERSION (ONLY CLASSES AND PACKAGES): 

?- enumerate_term_cache_for_all_classes. 
% 784,022,939 inferences, 268.41 CPU in 349.71 seconds (77% CPU, 2921031 Lips)           

TIME OF PREVIOUS VERSION (DIFFERENT VARIANTS, SEE BELOW):

?- time(fill_term_cache).

PACKAGES, CLASSES, METHODS AND FIELDS, WITH DUPLICATES:

227528  term cache entries for context_has_term/3.
125390  term cache entries for cached_occurrences_of_term_in_context/3.
125390  term cache entries for cached_occurrences_of_all_terms_in_context/2.
 80284  term cache entries for ri_element_has_lexical/2.
   324  term cache entries for context_has_introduced_term/2.
% 17,763,110 inferences, 27.41 CPU in 27.45 seconds (100% CPU, 648141 Lips)
true.

PACKAGES, CLASSES, METHODS AND FIELDS, NO DUPLICATES:

term_cache: 118 ?- time(fill_term_cache).
227528  term cache entries for context_has_term/3.
 89168  term cache entries for cached_occurrences_of_term_in_context/3.
 20626  term cache entries for cached_occurrences_of_all_terms_in_context/2.
 80284  term cache entries for ri_element_has_lexical/2.
   227  term cache entries for context_has_introduced_term/2.
% 16,215,000 inferences, 27.84 CPU in 27.94 seconds (100% CPU, 582357 Lips)
true.

 time(fill_term_cache).
114369  term cache entries for context_has_term/3.
 29412  term cache entries for cached_occurrences_of_term_in_context/3.
  3073  term cache entries for cached_occurrences_of_all_terms_in_context/2.
 80284  term cache entries for ri_element_has_lexical/2.
  3073  term cache entries for relevant_context_element/1.
   227  term cache entries for context_has_introduced_term/2.
% 12,440,799 inferences, 10.91 CPU in 10.95 seconds (100% CPU, 1140704 Lips)


ONLY PACKAGES AND CLASSES, WITH DUPLICATES:

term_cache: 114 ?- time(fill_term_cache).
157322  term cache entries for context_has_term/3.
 55266  term cache entries for cached_occurrences_of_term_in_context/3.
 55266  term cache entries for cached_occurrences_of_all_terms_in_context/2.
 80284  term cache entries for ri_element_has_lexical/2.
   324  term cache entries for context_has_introduced_term/2.
% 14,456,834 inferences, 26.38 CPU in 26.44 seconds (100% CPU, 548126 Lips)
true. 
ONLY PACKAGES AND CLASSES, NO DUPLICATES:

term_cache: 116 ?- time(fill_term_cache).
157322  term cache entries for context_has_term/3.
 37332  term cache entries for cached_occurrences_of_term_in_context/3.
  1596  term cache entries for cached_occurrences_of_all_terms_in_context/2.
 80284  term cache entries for ri_element_has_lexical/2.
   227  term cache entries for context_has_introduced_term/2.
% 13,339,194 inferences, 26.17 CPU in 26.42 seconds (99% CPU, 509677 Lips)
true.	


JHotdraw
generate_term_id_mapping
4,297,911 inferences, 0.875 CPU in 0.941 seconds (93% CPU, 4911898 Lips)
generate_relevant_context_elements
212,210 inferences, 0.109 CPU in 0.106 seconds (103% CPU, 1940206 Lips)
generate_term_context_mapping
5,549,994 inferences, 1.484 CPU in 1.542 seconds (96% CPU, 3738943 Lips)
3,312,236 inferences, 0.984 CPU in 0.999 seconds (99% CPU, 3364811 Lips)

report statistics:
598,771 inferences, 0.281 CPU in 0.275 seconds (102% CPU, 2128964 Lips)
*/  
