/**
 * These two predicates are temporariliy borrowed from the CTC
 * (file .../ct_core/utils/utils4modules.pl)
 * When integrating Cultivate with the CTC (or running both
 * together) this file should NOT be loaded to avoid name clashes.
 * GK -- 31.03.2009
 */

/**
 * declared_in_module(+Module, +Head) is det.
 * declared_in_module(+Module, ?Head) is nondet.
 * declared_in_module(?Module, ?Head) is nondet.
 *  
 * Determine whether the predicate indicated by Head has a *local*
 * (non-imported) declaration in the module Module. The predicate 
 * succeeds if there is at least a declaration (but possibly no  
 * single clause) of the predicate in the module. If you want it to  
 * fail you must abolish the predicate declaration, not just  
 * retract all its clauses. See the built-ins Prolog predicates   
 * abolish/1, abolish/1, retractall/1, retract/1
 */  
declared_in_module(Module, Head) :-
   current_predicate(_, Module:Head),                    % Head is declared
   \+ predicate_property(Module:Head, imported_from(_)). % but not imported
       
/**
 * defined_in_module(+Module, +Head, ?NrOfClauses) is det.
 *  
 * Determine whether the predicate indicated by Head has a local
 * (non-imported) definition in the module Module. The predicate 
 * succeeds if there is at least one clause of the predicate in 
 * the module and the number of clauses is unifiable with  
 * ?NrOfClauses. 
 */
 
defined_in_module(Module, Head) :-
   defined_in_module(Module, Head, _).  
   
defined_in_module(Module, Head, N) :-
    declared_in_module(Module, Head),
    % once(clause_in_module(Module,Head,_)),
    Module:predicate_property(Head, number_of_clauses(N)),
    N>0. 

    
:- module_transparent count/2.

count(Goal, _) :-
  flag(successcounter,_,0),
  % call(Goal),
  catch(Goal,_Any,fail),     % turn exceptions into failures
  flag(successcounter,N,N+1),
  fail.
count(_, N) :-
  flag(successcounter,N,N).