:- multifile query_description/3.
:- multifile metric_/3.
:- multifile metric_attributes/2.

query_description(afferentCouplings, metric, 'Number of afferent coupelings, that means the number of classes outside of this package that depend on classes inside of this package').
metric_attributes(afferentCouplings, int(range(0, infinity), defaultthresholds(0, infinity))).
             
/**
 * afferentCouplings(+PackageId, AfferentCoupling)
 *
 * Returns the number of non-external (!!!) classes outside the package
 * PackageId/PackageName that depend on classes inside the package.
 * For external packages the predicate fails.
 */
metric_(afferentCouplings, PackageID, AfferentCouplings) :-
    internal_package(PackageID),
    findall( Class,
             afferentDependencies(PackageID, Class),
             List),
    sort(List,DuplicateFree),
    length(DuplicateFree,AfferentCouplings).

/**
 * afferentDependencies(+Owner,?ClassID)
 *
 *  Succeeds if the non-external(!!!) class / interface 'ClassID'
 *  depends on a class / interface inside the Package 'Owner'.
 */
afferentDependencies(Package,ReferencingClassID) :-
    % modified for JT 2.6, old usage was  only ri_classT_parent(Package,ClassInPackageID), 
    ri_compilationUnitT_parent(Package, CompilationUnit),
    ri_classT_parent(CompilationUnit, ClassInPackageID),
    packageT(OtherPackage,_name),
    not(OtherPackage = Package),
    ri_compilationUnitT_parent(OtherPackage, OtherCompilationUnit),
    ri_classT_parent(OtherCompilationUnit, ReferencingClassID),
    not(externT(ReferencingClassID)),%%% <-- Do we want this??? - GK    
	classDependency(ReferencingClassID, ClassInPackageID,_).
	